@@ -1,5 +1,5 @@
 /*  GNU ddrescue - Data recovery tool
-    Copyright (C) 2004-2019 Antonio Diaz Diaz.
+    Copyright (C) 2004-2020 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@ int verbosity = 0;
 
 namespace {
 
-const char * const program_year = "2019";
+const char * const program_year = "2020";
 std::string command_line;
 
 
@@ -114,10 +114,10 @@ bool check_types( std::string & types, const char * const opt_name,
     if( !Sblock::isstatus( types[i] ) )
       { error = true; break; }
     }
-  if( !types.size() || error )
+  if( types.empty() || error )
     {
     char buf[80];
-    snprintf( buf, sizeof buf, "Invalid type for '%s' option.", opt_name );
+    snprintf( buf, sizeof buf, "Invalid type for option '%s'.", opt_name );
     show_error( buf, 0, true );
     std::exit( 1 );
     }
@@ -273,7 +273,7 @@ const char * format_num( long long num, long long limit,
 
 
 // Shows the fraction "num/den" as a percentage with "prec" decimals.
-// If 'prec' is negative, only the needed decimals are shown.
+// If 'prec' is negative, only the decimals needed are shown.
 //
 const char * format_percentage( long long num, long long den,
                                 const int iwidth, int prec )
@@ -304,7 +304,7 @@ const char * format_percentage( long long num, long long den,
       {
       buf[i++] = '.';
       while( prec > 0 && ( rest > 0 || !trunc ) && i < sizeof( buf ) - 2 )
-        { rest *= 10; buf[i++] = (char)( rest / den ) + '0';
+        { rest *= 10; buf[i++] = ( rest / den ) + '0';
           rest %= den; --prec; }
       }
     }
